; example for the controller-board (www.frydaysoft.de)
; print clock to LCD mm:ss:xx  (xx=milli_sec)
; count with T0=10ms

$include (mod51)

;**** Bit-RAM 20..2F ****
 BSEG AT 02Fh		; use only last byte of BSEG-block     
  blick_cursor: DBIT 1	; param for init_lcd
  step_walk:    DBIT 1  ; param for writeLCD_Array
  clock_inced:  DBIT 1  ; sign T0 step

;**** Byte-RAM from 8h upto 2Dh ****
 DSEG AT 8h		; bank1..3 8..1Fh
  min:          DS  1h   ; mm:ss:xx (m=min, s=sec, x=msec.)
  sec:          DS  1h   ;
  msec:         DS  1h   ;
  unused_buf:   DS 15h	; 16+5 byte unused and BSEG not complete used
 DSEG AT 30h		; 30..7Fh
  STACK_RAM:   DS 50h	; **** reserved STACK-area  ****


;**** Reg-zuweisung ****
Key4   SET   P1.0
Key3   SET   P1.1
Key2   SET   P1.2
Key1   SET   P1.3
Led4   SET   P1.4
Led3   SET   P1.5
Led2   SET   P1.6
Led1   SET   P1.7

tmppointer	equ R0
;R1..R7 not used

Stack_First 	equ  30h     	;set stackspointer to reserved RAM
; 9600 , XTAL=11,059MHz -> Timer=250=fah
baud 		equ  250        ;timer1 -> baud=256-fosz/(Baud-rate*12*16)

;***Pins of HCT138 (IC6)***********************************************************************

LCD_RegWR 	equ  80h       	;write lcd-register A0=0 ,A1=0
LCD_RegRD 	equ  81h       	;read  lcd-register A0=1 ,A1=0
LCD_DataWR 	equ  82h       	;write lcd-data	    A0=0 ,A1=0
LCD_DataRD 	equ  83h       	;read  lcd-data	    A0=1 ,A1=0
HCT138_Y1	equ  10010000b
HCT138_Y2	equ  10100000b
HCT138_Y3	equ  10110000b
HCT138_Y4	equ  11000000b
HCT138_Y5	equ  11010000b
HCT138_Y6	equ  11100000b
HCT138_Y7	equ  11110000b


; FFFFh - (XTAL/12) / 10ms   	 ;at 11.0592 MHz  --> 56320 = DC00h
TH0Value  equ 0DCh               ;16BIT-value for 10ms-timer
TL0Value  equ 00h                

;##################### MAIN Program #######################################

cseg at 0
	;USING 3                 
	;USING 2                 
	;USING 1                 
	USING 0     	;Bank 0

org 0000h
jmp main_start		;jump to program

org 0003h               ;extInt0
reti

org 000bh               ;timer0
  jmp isrT0

org 0013h               ;extInt1
reti

org 001Bh               ;timer1
  reti

org 0023h               ;RS232
reti
  ;jmp isrSIO

;**************************************************************************

main_start:
	mov SP,  #Stack_First
	mov DPTR,#0
	mov acc, #0
	mov b,	 #0
	mov PSW, #0
	mov PCON,#0
	mov SCON,#0
	mov TCON,#0
	mov TMOD,#0
	mov IE,  #0
	mov IP,  #0

	clr step_walk
	clr blick_cursor

	call lcd_init   ; init LCD (in auto-cursor-position-mode)

	call init_com   ; init V.24 at 9600 Baud

	clr RI

	call init_T0	; init Timer0 for 10ms
	clr clock_inced ; return-bit from T0-intr

	mov  min,#00h	; mm:ss:xx   init clock
	mov  sec,#00h	
	mov msec,#00h	

	call print	; print clock

	setb EA		; enable all intr.

;################## Main - L O O P ########################

MainLoop:

	jnb clock_inced,MainLoop ; wait for T0-event
	clr clock_inced
	mov a,#0
	call setcursor		 ; constant print to LCD
	call print

	jmp MainLoop

; ------------- print clock to LCD
print:
	mov a,min
	call printBCD
	mov a,#':'
	call write_char		;write byte to LCD
	mov a,sec
	call printBCD
	mov a,#':'
	call write_char		;write byte to LCD
	mov a,msec
	call printBCD
	ret

; ------------- print acc (bcd-format) to LCD
printBCD:
	mov b,a
	swap a		
	anl a,#0Fh
	add a,#30h
	call write_char		;write byte to LCD
	mov a,b
	anl a,#0Fh	
	add a,#30h
	call write_char		;write byte to LCD
 	ret


;--------------------------------------------
;put byte to LCD
;acc -> data 
write_char:
	push dpl
	push dph
	mov dpl,#LCD_DataWR
	anl dpl,#0fh
	mov dph,#LCD_DataWR
	anl dph,#0f0h
	movx @DPTR,a
	pop dph
	pop dpl
	ret

;--------------------------------------------
;reads LCD-Contr. Carry=1 if busy
LCDBusy:        

	;xxl    anlernative code  -> only short waitcycle
	;call shortwait	;wait a little time
	;clr c		;LCD should be ready
	;ret

	push dpl
	push dph
	mov dpl,#LCD_RegRD
	anl dpl,#0fh
	mov dph,#LCD_RegRD
	anl dph,#0f0h
	movx a,@DPTR
	clr c
	anl a,#80h
	jz LCDBusyRet
	setb c
LCDBusyRet:
	pop dph
	pop dpl
	ret
	
;--------------------------------------------
; set cursor in LCD-module
; in acc
setcursor:
	push dpl
	push dph
	mov dpl,#LCD_RegWR
	anl dpl,#0fh
	mov dph,#LCD_RegWR
	anl dph,#0f0h
	orl a,#80h	;set command-bit
	movx @DPTR,a	;set cursor
	pop dph
	pop dpl
	ret

;--------------------------------------------
;read RAM-addr of LCD and return it in acc (0-127)
;out acc
getCursor:
	push dpl
	push dph
	mov dpl,#LCD_RegRD
	anl dpl,#0fh
	mov dph,#LCD_RegRD
	anl dph,#0f0h
	movx a,@DPTR
	anl a,#7Fh	;clear busy-flag
	pop dph
	pop dpl
	ret
;--------------------------------------------
; set cursor home and clear LCD-module
clearLCD:
	push dpl
	push dph
	mov a,#1h               ;homecursor and clear

	mov dpl,#LCD_RegWR
	anl dpl,#0fh
	mov dph,#LCD_RegWR
	anl dph,#0f0h
	movx @DPTR,a
clear_busy:
	call shortwait
	call LCDBusy
	jc clear_busy
	pop dph
	pop dpl
        ret

;--------------------------------------------
; wait a little bit
shortwait:
	push acc
	mov a,#0
shortwait0:
	nop
	nop
	nop
	djnz acc,shortwait0
	pop acc
	ret

;--------------------------------------------
;acc -> data 
;b   -> IO-high-adresse
out_io:
	push dpl
	push dph
	mov dpl,b
	anl dpl,#0fh
	mov dph,b
	anl dph,#0f0h
	movx @DPTR,a
	pop dph
	pop dpl
	ret

;--------------------------------------------
;return data in acc 
;b   -> IO-high-adresse
in_io:
	push dpl
	push dph
	mov dpl,b
	anl dpl,#0fh
	mov dph,b
	anl dph,#0f0h
	movx a,@DPTR
	pop dph
	pop dpl
	ret

;----------- init LCD-modul ------------------------------------------
lcd_init:
	mov tmppointer,#2
init_again:
	call shortwait
	mov a,#00111000b		;8bit-mode one line 5x7dots
	mov b,#LCD_RegWR
	lcall out_io
	lcall out_io
	call shortwait
	djnz tmppointer,init_again	;call 2x init-sequence

	jb blick_cursor,b_cursor	
	mov a,#00001100b      		;LCD on no Cursor
	jmp after_cur
b_cursor:
	mov a,#00001101b      		;LCD on + Cursor
after_cur:
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	mov a,#00000001b		;clear lcd
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	mov a,#00000110b		;set corsor-direction auto-dec
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	clr blick_cursor
	ret


;----------- init T0 for 10ms ------------------------------------------
init_T0:
	clr TR0			;stop T0
	anl TMOD,#11110000b     ;16 bit Timer0 (gate C/T MODE)
	orl TMOD,#00000001b     ;Gate0 (01-16-bit 10-8bit reload)
	mov TL0,#TL0Value
	mov TH0,#TH0Value 

	setb ET0		;enable T0
	clr  TF0	        ;clr T0 overflow-bit
        setb TR0		;start timer

	ret


;--------------------------------------------
;init RS232
init_com:
	orl PCON,#80h      ; normal or faster processing
	orl SCON,#50h      ; SCON-Mode 1 (8Bit-UART) and REN && SM1
	orl TMOD,#20h      ; set 8Bit-auto-reload-TIMER0  (00100000)
	mov TH1,#baud      ; init Timer1 for Baud-rate
	mov TL1,#baud

	setb SM2
	clr TF1		   ;clr T1 overflow-bit
	setb ET1	   ;enable T1
	setb TR1	   ;start T1

        setb ES		   ;enable RS232-intr.
	clr RI             ;clr Receive-bit
	clr TI		   ;clr Transmit-bit
	clr PS		   ;lower priority for RS232-intr.
	ret                


;################ Interrupts ###################

isrSIO:
       jnb RI,Int_SIOExit
       clr RI
       ;push acc
       ;mov a,SBUF
       ;do something with accu
       ;pop acc
Int_SIOExit:
        reti

;----------- count clock (10ms-inc) ------------------------------------------

isrT0:
	push PSW
	push acc

	inc msec		;bcd 00..99
	mov a, msec
	clr c
	DA a
	mov msec,a		;write back
	jnc ret_isrT0		
	mov msec,#0
	inc sec		    	;bcd 00..59
	mov a,sec
	clr c
	DA a
	mov sec,a		;write back
	cjne a,#60h,ret_isrT0
	mov sec,#0
	inc min			;bcd 00..59
	mov a,min
	clr c
	DA a
	mov min,a		;write back
	cjne a,#60h,ret_isrT0
	mov min,#0
ret_isrT0:
	setb clock_inced
	pop acc
	pop PSW
	clr TR0			;stop T0
	mov TL0,#TL0Value	;reinit T0
	mov TH0,#TH0Value 
	setb TR0		;run T0
	reti

;****************************************************************

end
